@@ -527,13 +527,28 @@ impl<'data> ModuleEnvironment<'data> {
                     let mut elements =
                         Vec::with_capacity(usize::try_from(items_reader.get_count()).unwrap());
                     for item in items_reader {
-                        elements.push(match item? {
-                            ElementItem::Func(f) => {
+                        let func = match item? {
+                            ElementItem::Func(f) => Some(f),
+                            ElementItem::Expr(init) => {
+                                match init.get_binary_reader().read_operator()? {
+                                    Operator::RefNull { .. } => None,
+                                    Operator::RefFunc { function_index } => Some(function_index),
+                                    s => {
+                                        return Err(WasmError::Unsupported(format!(
+                                            "unsupported init expr in element section: {:?}",
+                                            s
+                                        )));
+                                    }
+                                }
+                            }
+                        };
+                        elements.push(match func {
+                            Some(f) => {
                                 let f = FuncIndex::from_u32(f);
                                 self.flag_func_escaped(f);
                                 f
                             }
-                            ElementItem::Null(_ty) => FuncIndex::reserved_value(),
+                            None => FuncIndex::reserved_value(),
                         });
                     }
 
@@ -1262,6 +1277,11 @@ and for re-adding support for interface types you can see this issue:
                     let mut names = f.get_map()?;
                     for _ in 0..names.get_count() {
                         let Naming { index, name } = names.read()?;
+                        // Skip this naming if it's naming a function that
+                        // doesn't actually exist.
+                        if (index as usize) >= self.result.module.functions.len() {
+                            continue;
+                        }
                         let index = FuncIndex::from_u32(index);
                         self.result
                             .module
@@ -1290,6 +1310,11 @@ and for re-adding support for interface types you can see this issue:
                     let mut reader = l.get_indirect_map()?;
                     for _ in 0..reader.get_indirect_count() {
                         let f = reader.read()?;
+                        // Skip this naming if it's naming a function that
+                        // doesn't actually exist.
+                        if (f.indirect_index as usize) >= self.result.module.functions.len() {
+                            continue;
+                        }
                         let mut map = f.get_map()?;
                         for _ in 0..map.get_count() {
                             let Naming { index, name } = map.read()?;
